﻿namespace MetaTrader {

	using System;
	using System.Reflection;
	using XNSNET.Collections;

	public class ClasterMonoCollection: TemplatedList<ClasterMonoCollection, ClasterMono> {

		internal ClasterMonoCollection() : base() {
			ClasterMono _USD = Append( "USD" );
			ClasterMono _EUR = Append( "EUR" );
			ClasterMono _JPY = Append( "JPY" );
			ClasterMono _CHF = Append( "CHF" );
			ClasterMono _GBP = Append( "GBP" );
		}

		public ClasterMono this[ string iName ] {
			get {
				ClasterMono _Next = this.First;
				ClasterMono _Item;
				while ( _Next != null ) {
					_Item = _Next;
					_Next = _Next.Next;
					if ( _Item.Name == iName )
						return _Item;
				}
				return null;
			}
		}

		public ClasterMono Append( string iName ) {
			if ( iName == null )
				throw new ArgumentNullException();
			ClasterMono _Item = this[ iName ];
			if ( _Item != null ) return _Item;
			return AddFinal( new ClasterMono( iName ) );
		}

		public ClasterMono Remove( string iName ) {
			if ( iName == null )
				throw new ArgumentNullException();
			ClasterMono _Item = this[ iName ];
			if ( _Item != null ) {
				Remove( _Item );
			}
			return _Item;
		}
		
		public void AppendClaster( Claster iClaster ) {
			if ( iClaster == null ) return;
			if ( iClaster.Up != null && iClaster.Down != null ) return;
			ClasterMono _Next = this.First;
			ClasterMono _Item;
			ClasterMono _Up = null;
			ClasterMono _Down = null;
			while ( _Next != null && ( _Up == null || _Down == null ) ) {
				_Item = _Next;
				_Next = _Next.Next;
				if ( iClaster.Symbol.StartsWith( _Item.Name ) ) {
					_Up = _Item;
				} else if ( iClaster.Symbol.EndsWith( _Item.Name ) ) {
					_Down = _Item;
				}
			}
			if ( _Up != null && _Down != null ) {
				_Down.Pairs.Append( _Up ).Claster = iClaster;
				_Up.Pairs.Append( _Down ).Claster = iClaster;
				iClaster.Up = _Up;
				iClaster.Down = _Down;
			}
		}

		public void RemoveClaster( Claster iClaster ) {
			if ( iClaster == null ) return;
			if ( iClaster.Up == null && iClaster.Down == null ) return;
			ClasterMono _Up = iClaster.Up;
			ClasterMono _Down = iClaster.Down;
			_Up.Pairs.Remove( _Down );
			_Down.Pairs.Remove( _Up );
			iClaster.Up = null;
			iClaster.Down = null;
		}

		public ClasterMono[] Array {
			get {
				ClasterMono[] _Array = new ClasterMono[ Count ];
				int _Index = 0;
				ClasterMono _Item = this.First;
				while ( _Item != null ) {
					_Array[ _Index ] = _Item;
					_Item = _Item.Next;
					_Index++;
				}
				return _Array;
			}
		}

	}

	public class ClasterMono: TemplatedItem<ClasterMonoCollection, ClasterMono> {

		public readonly string Name;
		public ClasterPairCollection Pairs;

		internal ClasterMono( string iName )
			: base() {
			Name = iName;
			Pairs = new ClasterPairCollection( this );
		}

		public override void OnRemoved() {
			Pairs.Clear();
			ClasterMono _Next = Global.Compare.First;
			ClasterMono _Item;
			while ( _Next != null ) {
				_Item = _Next;
				_Next = _Next.Next;
				_Item.Pairs.Remove( this );
			}
		}

		public override string ToString() {
			return Name;
		}

	}

}