﻿namespace MetaTrader {

	using System;
	using System.Reflection;
	using XNSNET.Collections;

	public class ClasterPairCollection: TemplatedList<ClasterPairCollection, ClasterPair> {

		public readonly ClasterMono Mono;

		internal ClasterPairCollection( ClasterMono iMono )
			: base() {
			Mono = iMono;
		}

		public ClasterPair this[ ClasterMono iMono ] {
			get {
				ClasterPair _Next = this.First;
				ClasterPair _Item;
				while ( _Next != null ) {
					_Item = _Next;
					_Next = _Next.Next;
					if ( _Item.Mono == iMono )
						return _Item;
				}
				return null;
			}
		}

		public ClasterPair Append( ClasterMono iMono ) {
			if ( iMono == null )
				throw new ArgumentNullException();
			if ( iMono == Mono )
				throw new ArgumentException();
			ClasterPair _Item = this[ iMono ];
			if ( _Item != null ) return _Item;
			return AddFinal( new ClasterPair( iMono ) );
		}

		public ClasterPair Remove( ClasterMono iMono ) {
			if ( iMono == null )
				throw new ArgumentNullException();
			ClasterPair _Item = this[ iMono ];
			if ( _Item != null ) {
				Remove( _Item );
			}
			return _Item;
		}

		public ClasterPair[] Array {
			get {
				ClasterPair[] _Array = new ClasterPair[ Count ];
				int _Index = 0;
				ClasterPair _Item = this.First;
				while ( _Item != null ) {
					_Array[ _Index ] = _Item;
					_Item = _Item.Next;
					_Index++;
				}
				return _Array;
			}
		}

	}

	public class ClasterPair: TemplatedItem<ClasterPairCollection, ClasterPair> {

		public readonly ClasterMono Mono;
		public Claster Claster;

		internal ClasterPair( ClasterMono iMono )
		  : base() {
			Mono = iMono;
			Claster = null;
		}

		public override void OnRemoved() {
			if ( Claster != null ) {
				Global.Compare.RemoveClaster( Claster );
				Claster = null;
			}
		}

	}

}