﻿namespace MetaTrader {

	using System;
	using System.Reflection;
	using XNSNET.Collections;

	public class ClasterTickCollection: TemplatedList<ClasterTickCollection, ClasterTick> {

		internal readonly Claster Claster;
		internal int MaxCount = 999999;

		internal ClasterTickCollection( Claster iClaster, int iMaxCount )
			: base() {
			Claster = iClaster;
			if ( iMaxCount > 0 && iMaxCount < 1000000 )
				MaxCount = iMaxCount;
		}

		public ClasterTick AddValue( DateTime iUtc, double iBid, DateTime iSrv ) {
			while ( Count >= MaxCount ) {
				Remove( First );
			}
			return AddFinal( new ClasterTickValue( iUtc, iSrv, iBid ) );
		}

		public ClasterTick AddSpace( DateTime iUtc ) {
			while ( Count >= MaxCount ) {
				Remove( First );
			}
			return AddFinal( new ClasterTickSpace( iUtc ) );
		}

	}

	public class ClasterTick: TemplatedItem<ClasterTickCollection, ClasterTick> {

		public readonly DateTime CalledStamp;

		public double Ask {
			get {
				return Bid + ( List.Claster.Point * List.Claster.Spread );
			}
		}

		public virtual double Bid {
			get {
				throw new Exception( "Undefined property" );
			}
		}

		public double AskInt {
			get {
				return ( Bid + ( List.Claster.Point * List.Claster.Spread ) ) * ( Math.Pow( 10, ( List.Claster.Digits - 1 ) ) );
			}
		}

		public double BidInt {
			get {
				return Bid * ( Math.Pow( 10, (List.Claster.Digits - 1) ) );
			}
		}
		
		public virtual DateTime ServerStamp {
			get {
				throw new Exception( "Undefined property" );
			}
		}

		protected ClasterTick( DateTime iCalledStamp )
			: base() {
			CalledStamp = iCalledStamp;
		}

	}

	public class ClasterTickSpace: ClasterTick {

		public ClasterTickSpace( DateTime iCalledStamp )
			: base( iCalledStamp ) {
			
		}
		
	}

	public class ClasterTickValue: ClasterTick {

		public readonly DateTime mServerStamp;
		public readonly double mBid;

		public ClasterTickValue( DateTime iCalledStamp, DateTime iServerStamp, double iBid )
			: base( iCalledStamp ) {
			mServerStamp = iServerStamp;
			mBid = iBid;
		}

		public override double Bid {
			get {
				return mBid;
			}
		}

		public override DateTime ServerStamp {
			get {
				return mServerStamp;
			}
		}
		
	}

}