﻿/******************************************************************************\
   Все права (c) XNSNET 2005 - 2007 & Автор Иванов M.A. [Shum.er] 2005 - 2007 
\******************************************************************************/
namespace XNSNET.Collections {

	using System;

	/// <summary>
	/// Астрактный класс объектного элемента связанного списка.
	/// </summary>
	/// <example>
	/// </example>
	/// <remarks>
	/// Используется для элементарного связывания элементов списка, где все элементы
	/// включая сам список между собой взаимосвязаны, без дополнительного распределения
	/// новых блоков памяти для включения в список новых элементов или для удаления.
	/// </remarks>
	public abstract class ObjectedItem {
	
		private ObjectedList mList;
		private ObjectedItem mPrev;
		private ObjectedItem mNext;

		protected ObjectedItem() {
			Invalidate();
		}

		/// <summary>
		/// Возвращает экземпляр списка владеющего этим элементом.
		/// </summary>
		public ObjectedList List {
			get {
				return mList;
			}
			internal set {
				if ( mList != null || value == null )
					throw new ArgumentNullException();
				mList = value;
			}
		}

		/// <summary>
		/// Возвращает приведущий элемент в связанном списке.
		/// </summary>
		public ObjectedItem Prev {
			get {
				return mPrev;
			}
			internal set {
				if ( mPrev == null && value == null )
					throw new ArgumentNullException();
				mPrev = value;
			}
		}

		/// <summary>
		/// Возвращает следующий элемент в связанном списке.
		/// </summary>
		public ObjectedItem Next {
			get {
				return mNext;
			}
			internal set {
				if ( mNext == null && value == null )
					throw new ArgumentNullException();
				mNext = value;
			}
		}

		internal void Invalidate() {
			mList = null;
			mPrev = null;
			mNext = null;
		}

		public bool IsInvalid() {
			return ( mList == null && mPrev == null && mNext == null );
		}

		public virtual ObjectedItem Remove() {
			if ( mList != null ) {
				return mList.Remove( this );
			}
			return null;
		}

		public virtual void OnAdded() {

		}

		public virtual void OnRemoved() {

		}
		
	}

}