﻿/******************************************************************************\
   Все права (c) XNSNET 2005 - 2007 & Автор Иванов M.A. [Shum.er] 2005 - 2007 
\******************************************************************************/
namespace XNSNET.Collections {

	using System;

	/// <summary>
	/// Астрактный класс шаблонного элемента связанного списка.
	/// </summary>
	/// <typeparam name="LIST"> Шаблонный тип списка связанных элементов. </typeparam>
	/// <typeparam name="ITEM"> Шаблонный тип элемента связанного списка. </typeparam>
	/// <example>
	/// </example>
	/// <remarks>
	/// Используется для элементарного связывания элементов списка, где все элементы
	/// включая сам список между собой взаимосвязаны, без дополнительного распределения
	/// новых блоков памяти для включения в список новых элементов или для удаления.
	/// </remarks>
	public abstract class TemplatedItem<LIST, ITEM>
		where LIST: TemplatedList<LIST, ITEM>
		where ITEM: TemplatedItem<LIST, ITEM> {

		private LIST mList;
		private ITEM mPrev;
		private ITEM mNext;
		
		protected TemplatedItem() {
			Invalidate();
		}

		/// <summary>
		/// Возвращает экземпляр списка владеющего этим элементом.
		/// </summary>
		public LIST List {
			get {
				return mList;
			}
			internal set {
				#region #исключения#
#if DEBUG
				if ( mList != null || value == null )
					throw new ArgumentNullException();
#endif
				#endregion #исключения#
				mList = value;
			}
		}

		/// <summary>
		/// Возвращает приведущий элемент в связанном списке.
		/// </summary>
		public ITEM Prev {
			get {
				return mPrev;
			}
			internal set {
				#region #исключения#
#if DEBUG
				if ( mPrev == null && value == null )
					throw new ArgumentNullException();
#endif
				#endregion #исключения#
				mPrev = value;
			}
		}

		/// <summary>
		/// Возвращает следующий элемент в связанном списке.
		/// </summary>
		public ITEM Next {
			get {
				return mNext;
			}
			internal set {
				#region #исключения#
#if DEBUG
				if ( mNext == null && value == null )
					throw new ArgumentNullException();
#endif
				#endregion #исключения#
				mNext = value;
			}
		}

		internal void Invalidate() {
			mList = null;
			mPrev = null;
			mNext = null;
		}

		internal bool IsInvalid() {
			return ( mList == null && mPrev == null && mNext == null );
		}

		public virtual ITEM Remove() {
			if ( mList != null ) {
				return mList.Remove( (ITEM)this );
			}
			return null;
		}
		
		public virtual void OnAdded() {
		
		}

		public virtual void OnRemoved() {

		}
		
	}

}