﻿/******************************************************************************\
   Все права (c) XNSNET 2005 - 2007 & Автор Иванов M.A. [Shum.er] 2005 - 2007 
\******************************************************************************/
namespace XNSNET.Collections {

	#region :- Импорт -:

	using System;

	#endregion :- Импорт -:

	public abstract class TemplatedNode<NODE>
		where NODE: TemplatedNode<NODE> {
	
    private int mCount;
		private NODE mFirst;
		private NODE mFinal;

		private NODE mList;
		private NODE mPrev;
		private NODE mNext;

		protected TemplatedNode() {
      Clearing();
      Invalidate();
    }

    private void Clearing() {
      mCount = 0;
      mFirst = null;
      mFinal = null;
    }

    public int Count {
      get {
        return mCount;
      }
      private set {
        #region #исключения#
#if DEBUG
        if ( value < 1 || value == mCount )
          throw new ArgumentOutOfRangeException();
#endif
        #endregion #исключения#
        mCount = value;
      }
    }

		public NODE First {
      get {
        return mFirst;
      }
      private set {
        #region #исключения#
#if DEBUG
        if ( value == null || value == mFirst )
          throw new ArgumentNullException();
#endif
        #endregion #исключения#
        mFirst = value;
      }
    }

		public NODE Final {
      get {
        return mFinal;
      }
      private set {
        #region #исключения#
#if DEBUG
        if ( value == null || value == mFinal )
          throw new ArgumentNullException();
#endif
        #endregion #исключения#
        mFinal = value;
      }
    }

		public virtual void Clear() {
      if ( this.Count < 1 )
        return;
      lock ( this ) {
				NODE _Next = this.First;
				NODE _Item;
        while ( _Next != null ) {
          _Item = _Next;
          _Next = _Next.Next;
          _Item.Invalidate();
          _Item.OnRemoved();
        }
        this.Clearing();
      }
    }

		protected NODE AddFinal( NODE iItem ) {
      if ( iItem == null )
        throw new ArgumentNullException();
      if ( !iItem.IsValidate() )
        throw new ArgumentException();
      lock ( this ) {
				iItem.List = (NODE)this;
        if ( this.Count == 0 ) {
          this.First = iItem;
        } else {
          iItem.Prev = this.Final;
          this.Final.Next = iItem;
        }
        this.Final = iItem;
				this.Count++;
				iItem.OnAdded();
				return iItem;
      }
    }

		protected NODE AddFirst( NODE iItem ) {
      #region #исключения#
#if DEBUG
      if ( iItem == null )
        throw new ArgumentNullException();
      if ( !iItem.IsValidate() )
        throw new ArgumentException();
#endif
      #endregion #исключения#
      lock ( this ) {
				iItem.List = (NODE)this;
        if ( this.Count == 0 ) {
          this.Final = iItem;
        } else {
          iItem.Next = this.First;
          this.First.Prev = iItem;
        }
        this.First = iItem;
				this.Count++;
				iItem.OnAdded();
				return iItem;
      }
    }

		protected NODE AddAfter( NODE iTodo, NODE iItem ) {
      #region #исключения#
#if DEBUG
      if ( iTodo == null || iItem == null )
        throw new ArgumentNullException();
      if ( iTodo.IsValidate() || !iItem.IsValidate() )
        throw new ArgumentException();
#endif
      #endregion #исключения#
      lock ( this ) {
				iItem.List = (NODE)this;
        iItem.Prev = iTodo;
        if ( iTodo.Next == null ) {
          this.Final = iItem;
        } else {
          iItem.Next = iTodo.Next;
          iTodo.Next.Prev = iItem;
        }
        iTodo.Next = iItem;
				this.Count++;
				iItem.OnAdded();
				return iItem;
      }
    }

		protected NODE AddBefore( NODE iTodo, NODE iItem ) {
      #region #исключения#
#if DEBUG
      if ( iTodo == null || iItem == null )
        throw new ArgumentNullException();
      if ( iTodo.IsValidate() || !iItem.IsValidate() )
        throw new ArgumentException();
#endif
      #endregion #исключения#
      lock ( this ) {
				iItem.List = (NODE)this;
        iItem.Next = iTodo;
        if ( iTodo.Prev == null ) {
          this.First = iItem;
        } else {
          iItem.Next = iTodo;
          iTodo.Prev.Next = iItem;
        }
        iTodo.Prev = iItem;
        this.Count++;
				iItem.OnAdded();
				return iItem;
      }
    }

		public virtual NODE Remove( NODE iItem ) {

      #region #исключения#
#if DEBUG
      if ( iItem == null )
        throw new ArgumentNullException();
#endif
      #endregion #исключения#

      if ( this.Count == 1 ) {
        this.Clearing();
      } else {
				lock ( this ) {
					if ( iItem.Prev != null && iItem.Next != null ) {
						iItem.Prev.Next = iItem.Next;
						iItem.Next.Prev = iItem.Prev;
					} else if ( iItem.Prev == null ) {
						this.First = iItem.Next;
						iItem.Next.Prev = null;
					} else if ( iItem.Next == null ) {
						this.Final = iItem.Prev;
						iItem.Prev.Next = null;
					}
					Count--;
        }
      }
			iItem.OnRemoved();
      iItem.Invalidate();
			return iItem;

    }

		/// <summary>
		/// Возвращает экземпляр списка владеющего этим элементом.
		/// </summary>
		public NODE List {
			get {
				return mList;
			}
			private set {
				#region #исключения#
#if DEBUG
				if ( mList != null || value == null )
					throw new ArgumentNullException();
#endif
				#endregion #исключения#
				mList = value;
			}
		}

		/// <summary>
		/// Возвращает приведущий элемент в связанном списке.
		/// </summary>
		public NODE Prev {
			get {
				return mPrev;
			}
			private set {
				#region #исключения#
#if DEBUG
				if ( mPrev == null && value == null )
					throw new ArgumentNullException();
#endif
				#endregion #исключения#
				mPrev = value;
			}
		}

		/// <summary>
		/// Возвращает следующий элемент в связанном списке.
		/// </summary>
		public NODE Next {
			get {
				return mNext;
			}
			private set {
				#region #исключения#
#if DEBUG
				if ( mNext == null && value == null )
					throw new ArgumentNullException();
#endif
				#endregion #исключения#
				mNext = value;
			}
		}

		private void Invalidate() {
			mList = null;
			mPrev = null;
			mNext = null;
		}

		private bool IsValidate() {
			return ( mList == null && mPrev == null && mNext == null );
		}

		public virtual NODE Remove() {
			if ( mList != null ) {
				return mList.Remove( (NODE)this );
			}
			return null;
		}

		public virtual void OnAdded() {
		}

		public virtual void OnRemoved() {
		}

		//public NODE this[ int iIndex ] {
		//  get {
		//    string place = null;
		//    lock ( this ) {
		//      NODE _Next = this.First;
		//      NODE _Item;
		//      int _Index = 0;
		//      while ( _Next != null ) {
		//        _Item = _Next;
		//        _Next = _Next.Next;
		//        if ( _Index == iIndex )
		//          return _Item;
		//        _Index++;
		//      }
		//    }
		//    return null;
		//  }
		//}
		

	}
	
}