using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace MetaTrader.Windows {
	public partial class FormDiagram: Form {
		
		public readonly Claster Claster;

		private int OldWidth;
		private int OldHeight;
		private int NowWidth;
		private int NowHeight;
		private Bitmap WorkBuffer;
		private Graphics WorkOutput;
		
		public FormDiagram( Claster iClaster ) {
			Claster = iClaster;
			NowWidth = OldWidth = Screen.PrimaryScreen.WorkingArea.Width;
			NowHeight = OldHeight = Screen.PrimaryScreen.WorkingArea.Height;
			WorkBuffer = new Bitmap( OldWidth, OldHeight );
			WorkOutput = Graphics.FromImage( WorkBuffer );
			InitializeComponent();
			Text = "Diagram " + iClaster.Symbol;
		}

		protected override void OnResize( EventArgs e ) {
			Application.DoEvents();
			if ( ClientSize.Width > OldWidth || ClientSize.Height > OldHeight ) {
				if ( ClientSize.Width > OldWidth ) OldWidth = ClientSize.Width;
				if ( ClientSize.Height > OldHeight ) OldHeight = ClientSize.Height;
				WorkBuffer.Dispose();
				WorkBuffer = new Bitmap( OldWidth, OldHeight );
				WorkOutput.Dispose();
				WorkOutput = Graphics.FromImage( WorkBuffer );
			}
			NowWidth = ClientSize.Width - 1; NowHeight = ClientSize.Height - 1;
			WorkOutput.FillRectangle( Brushes.Black, 0, 0, OldWidth, OldHeight );
			OnDrawContent( WorkOutput, NowWidth, NowHeight );
			using( Graphics DcGraphics = Graphics.FromHwnd( this.Handle ) ) {
				DcGraphics.DrawImageUnscaled( WorkBuffer, 0, 0, NowWidth, NowHeight );
			}
			base.OnResize( e );
		}

		protected override void OnLoad( EventArgs e ) {
			Application.DoEvents();
			NowWidth = ClientSize.Width - 1; NowHeight = ClientSize.Height - 1;
			WorkOutput.FillRectangle( Brushes.Black, 0, 0, OldWidth, OldHeight );
			OnDrawContent( WorkOutput, NowWidth, NowHeight );
			using ( Graphics DcGraphics = Graphics.FromHwnd( this.Handle ) ) {
				DcGraphics.DrawImageUnscaled( WorkBuffer, 0, 0, NowWidth, NowHeight );
			}
			timer1.Enabled = true;
			base.OnLoad( e );
		}

		protected override void OnShown( EventArgs e ) {
			WindowState = FormWindowState.Maximized;
			Application.DoEvents();
			NowWidth = ClientSize.Width - 1; NowHeight = ClientSize.Height - 1;
			WorkOutput.FillRectangle( Brushes.Black, 0, 0, OldWidth, OldHeight );
			OnDrawContent( WorkOutput, NowWidth, NowHeight );
			using ( Graphics DcGraphics = Graphics.FromHwnd( this.Handle ) ) {
				DcGraphics.DrawImageUnscaled( WorkBuffer, 0, 0, NowWidth, NowHeight );
			}
			base.OnShown( e );
		}

		protected override void OnPaintBackground( PaintEventArgs e ) {
			e.Graphics.DrawImageUnscaled( WorkBuffer, 0, 0, NowWidth, NowHeight );
		}

		protected override void OnClosed( EventArgs e ) {
			WorkBuffer.Dispose();
			WorkBuffer = null;
			WorkOutput.Dispose();
			WorkOutput = null;
			base.OnClosed( e );
		}

		protected virtual void RedrawContent() {
			try {
			using ( Graphics DcGraphics = Graphics.FromHwnd( this.Handle ) ) {
				WorkOutput.FillRectangle( Brushes.Black, 0, 0, OldWidth, OldHeight );
				OnDrawContent( WorkOutput, NowWidth, NowHeight );
				DcGraphics.DrawImageUnscaled( WorkBuffer, 0, 0, NowWidth, NowHeight );
			}
			} catch {}
		}
		
		private delegate void ReadrawCallback();

		public void InvokeRedrawContent() {
			Invoke( new ReadrawCallback( RedrawContent ) );
		}
		
		protected virtual void OnDrawContent( Graphics iGraphics, int iWidth, int iHeight ) {

			if ( Claster.Ticks.Count == 0 ) return;

			double _BoxMiddle = (int)iHeight / 2;
			double _IndexX = iWidth;
			double _Price;
			
			ClasterTick _Next = Claster.Ticks.Final;
			ClasterTick _Prev = _Next.Prev;
			if ( _Next != null && _Next is ClasterTickValue ) {
				_Price = _Next.BidInt;
			} else if ( _Prev != null && _Prev is ClasterTickValue ) {
				_Price = _Prev.BidInt;
			} else {
				return;
			}
			while ( _Prev != null && _IndexX > 0 ) {
				if ( _Next is ClasterTickSpace ) {
					iGraphics.DrawLine( Pens.Red, (int)( _IndexX ), (int)( 1 ), (int)( _IndexX ), (int)( iHeight - 1 ) );
					_Next = _Prev;
					_Prev = _Prev.Prev;
					_IndexX--;
				} else if ( _Prev is ClasterTickSpace ) {
					iGraphics.DrawLine( Pens.Red, (int)( _IndexX - 1 ), (int)( 1 ), (int)( _IndexX - 1 ), (int)( iHeight - 1 ) );
					_Prev = _Prev.Prev;
					_IndexX--;
				} else {
					double _PriceNext = ( _BoxMiddle + ( _Next.BidInt - _Price ) * 2 );
					double _PricePrev = ( _BoxMiddle + ( _Prev.BidInt - _Price ) * 2 );
					iGraphics.DrawLine( Pens.Red, (int)( _IndexX - 1 ), (int)( _PricePrev ), (int)( _IndexX ), (int)( _PriceNext - 1 ) );
					_Next = _Prev;
					_Prev = _Prev.Prev;
					_IndexX--;
				}
			}
			
		}

		private void timer1_Tick( object sender, EventArgs e ) {
			RedrawContent();
			Application.DoEvents();
		}

	}
}