using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace MetaTrader.Windows {
	public partial class FormMain: Form {
		public FormMain() {
			InitializeComponent();
		}

		private void StateIcon_MouseDoubleClick( object sender, MouseEventArgs e ) {
			if ( this.WindowState != FormWindowState.Minimized ) {
				this.WindowState = FormWindowState.Minimized;
				this.Visible = false;
			} else {
				this.Visible = true;
				this.WindowState = FormWindowState.Maximized;
				this.Activate();
			}
		}

		protected override void OnLoad( EventArgs e ) {
			MainIcon.Visible = true;
			foreach ( Claster claster in Global.Claster.Array ) {
				ToolStripItem menu = MenuClaster.DropDownItems.Add( claster.Symbol, MainImages.Images[ 0 ] );
				menu.Click += new EventHandler( menu_Click );
			}
			StatusSymbolCount.Text = "Symbols: " + Global.Claster.Count.ToString();
			Global.Claster.StateEvents += Claster_StateEvents;
			base.OnLoad( e );
		}
		
		protected override void OnShown( EventArgs e ) {
			this.Visible = false;
			base.OnShown( e );
		}

		protected override void OnClosing( CancelEventArgs e ) {
			if ( Global.Context.Count != 0 ) {
				e.Cancel = true;
				this.WindowState = FormWindowState.Minimized;
				this.Visible = false;
			}
			base.OnClosing( e );
		}

		protected override void OnClosed( EventArgs e ) {
			MainIcon.Visible = false;
			Global.Claster.StateEvents -= Claster_StateEvents;
			Global.Window = null;
			base.OnClosed( e );
		}
		
		private void MenuInfoClasters_Click( object sender, EventArgs e ) {
			new WindowSymbols().Show( this );
		}

		private void MenuInfoCompare_Click( object sender, EventArgs e ) {
			new WindowCompare().Show( this );
		}

		private void MainMenu_ItemClicked( object sender, ToolStripItemClickedEventArgs e ) {

		}

		private void Claster_StateEvents( ClasterEventType iAction, Claster iSender ) {
			Invoke( new ClasterEventHand( ClasterStateEvents ), iAction, iSender );
		}

		private void ClasterStateEvents( ClasterEventType iAction, Claster iSender ) {
			if ( iAction == ClasterEventType.Added ) {
				ToolStripItem menu = MenuClaster.DropDownItems.Add( iSender.Symbol, MainImages.Images[ 0 ] );
				menu.Click += new EventHandler( menu_Click );
			} else if ( iAction == ClasterEventType.Removed ) {
				MenuClaster.DropDownItems.RemoveByKey( iSender.Symbol );
			}
			StatusSymbolCount.Text = "Symbols: " + Global.Claster.Count.ToString();
		}

		void menu_Click( object sender, EventArgs e ) {
			ToolStripItem menu = sender as ToolStripItem;
			if ( menu != null ) {
				Claster claster = Global.Claster[ menu.Text ];
				FormDiagram form = null;
				foreach( Form frm in MdiChildren ) {
					form = frm as FormDiagram;
					if ( form != null && form.Claster == claster ) {
						form.WindowState = FormWindowState.Maximized;
						form.Activate();
						return;
					}
				}
				form = new FormDiagram( claster );
				form.MdiParent = this;
				form.Show();
			}
		}
	}
}